/*
 * aevents.c - Apple Event processing.
 *
 * The required core events are the only events for which handlers are installed.
 * Even so, the Open Application handler doesn't do anything, and the Print
 * Documents handler only says that printing isn't supported.
 *
 * Since Open Document events can occur at any time, even while a file is
 * currently being processed, the handler doesn't open any files.  It simply
 * puts them on a queue, to be processed later.
 *
 * AddDocuments() puts a document on the document list.
 * PendingDocuments() returns the number of documents waiting in the queue.
 * RemoveDocument() returns the pathname of the first document in the list
 * and removes it from the list.
 *
 * Before entering the main event loop, the application processes pending Apple
 * Events.  This allows information for any documents dropped on the application
 * to be extracted immediately.  Then, if PendingDocuments() is non-zero, the
 * application knows it should process those documents and exit.  Otherwise it
 * enters the event loop waiting for user actions.
 */

# include	<stdio.h>

# include	<AppleEvents.h>

# include	"TransSkel.h"
# include	"TransDisplay.h"

# include	"rtf-mac.h"


typedef	struct DocName	DocName, **DNHandle;

struct DocName
{
	FSSpec		fss;
	DNHandle	nextDoc;
};

static DNHandle	docHead = (DNHandle) nil;
static DNHandle	docTail = (DNHandle) nil;


short
PendingDocuments (void)
{
DNHandle	doc = docHead;
short		count = 0;

	while (doc != (DNHandle) nil)
	{
		++count;
		doc = (**doc).nextDoc;
	}
	return (count);
}


/*
 * Add document to document queue
 */

void
AddDocument (FSSpec *fss)
{
DNHandle	doc;
Str255		str;

	doc = (DNHandle) NewHandle (sizeof (DocName));
	if (doc == (DNHandle) nil)
		DisplayCString ("cannot allocate document structure.\r");
	else
	{
		(**doc).fss = *fss;
		(**doc).nextDoc = (DNHandle) nil;
		if (docHead == (DNHandle) nil)
			docHead = docTail = doc;
		else
		{
			(**docTail).nextDoc = doc;
			docTail = doc;
		}
		DisplayCString ("Document queued: ");
		DisplayString (fss->name);
		DisplayLn ();
	}
}


/*
 * Pull next document off document list.  Return true if there is one,
 * false otherwise.
 */

Boolean
RemoveDocument (FSSpec *fss)
{
DNHandle	doc;

	if (docHead == (DNHandle) nil)
		return (false);
	*fss = (**docHead).fss;
	doc = docHead;
	docHead = (**docHead).nextDoc;
	if (docHead == (DNHandle) nil)
		docTail = (DNHandle) nil;
	DisposeHandle ((Handle) doc);
	DisplayCString ("Document dequeued: ");
	DisplayString (fss->name);
	DisplayLn ();
	return (true);
}


/*
 * Handlers for the Open Application, Quit Application, Open Documents,
 * and Print Documents Apple Events.
 *
 * Note that these are pascal functions.
 */

static pascal OSErr
AEOpenApp (AppleEvent *theAEvt, AppleEvent *replyEvt, long refCon)
{
	return (noErr);
}


static pascal OSErr
AEQuitApp (AppleEvent *theAEvt, AppleEvent *replyEvt, long refCon)
{
	RequestQuit ();
	if (appState->quitRequestPending)	/* request succeeded */
		return (noErr);
	else								/* request was cancelled by user */
		return (errAEEventNotHandled);
}


static pascal OSErr
AEOpenDoc (AppleEvent *theAEvt, AppleEvent *replyEvt, long refCon)
{
long		items, index;
FSSpec		myFSS;
AEDescList	docList;
AEKeyword	keywd;
DescType	returnedType;
long		actualSize;
FInfo		finderInfo;
OSErr		myErr;
WindowPtr	w;

	myErr = AEGetParamDesc (theAEvt, keyDirectObject, typeAEList, &docList);
	if (myErr != noErr)
		return (myErr);

	/* should check for missing required parameters here */

	if ((myErr = AECountItems (&docList, &items)) != noErr)
		return (myErr);
	for (index = 1; index <= items; index++)
	{
		myErr = AEGetNthPtr (&docList, index, typeFSS, &keywd, &returnedType,
						(Ptr) &myFSS, sizeof (myFSS), &actualSize);
		if (myErr != noErr)
			return (myErr);
		myErr = FSpGetFInfo (&myFSS, &finderInfo);
		if (myErr != noErr || finderInfo.fdType != docType)
		{
			DisplayString (myFSS.name);
			DisplayCString (": unknown or incorrect file type.");
			DisplayLn ();
		}
		else
		{
			AddDocument (&myFSS);
			StatusWindUpdate ();
		}
	}
	myErr = AEDisposeDesc (&docList);
	return (myErr);
}


static pascal OSErr
AEPrintDoc (AppleEvent *theAEvt, AppleEvent *replyEvt, long refCon)
{
	DisplayCString ("This application does not print.");
	DisplayLn ();
	return (noErr);
}


/*
 * Function called by TransSkel when a high-level event occurs
 */

static pascal void
DoAppleEvent (EventRecord *theEvent)
{
	AEProcessAppleEvent (theEvent);
}


/*
 * If Apple Events are available, install handlers for the required core
 * events.
 */

void
AppleEventsInit (void)
{
Handle	h;
long	result;

	if (!SkelQuery (skelQHasAppleEvents))
		return;
	SkelSetAEHandler (DoAppleEvent);
	if (AEInstallEventHandler (kCoreEventClass, kAEOpenApplication,
									AEOpenApp, 0L, false) != noErr)
		DisplayCString ("AEInstallEventHandler failed (Open Application)\r");
	if (AEInstallEventHandler (kCoreEventClass, kAEQuitApplication,
									AEQuitApp, 0L, false) != noErr)
		DisplayCString ("AEInstallEventHandler failed (Quit Application)\r");
	if (AEInstallEventHandler (kCoreEventClass, kAEOpenDocuments,
									AEOpenDoc, 0L, false) != noErr)
		DisplayCString ("AEInstallEventHandler failed (Open Documents)\r");
	if (AEInstallEventHandler (kCoreEventClass, kAEPrintDocuments,
									AEPrintDoc, 0L, false) != noErr)
		DisplayCString ("AEInstallEventHandler failed (Print Documents)\r");
}
